package com.armcloud.armcloudsdkdemo

import android.app.Activity
import android.app.AlertDialog
import android.content.ClipData
import android.content.ClipboardManager
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Bitmap
import android.os.Build
import android.os.Bundle
import android.provider.MediaStore
import android.text.TextUtils
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.RadioButton
import android.widget.RadioGroup
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.AppCompatButton
import androidx.appcompat.widget.AppCompatTextView
import androidx.appcompat.widget.LinearLayoutCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.lifecycle.lifecycleScope
import com.arm.armcloudsdk.ArmCloudEngine
import com.arm.armcloudsdk.bean.LocationInfo
import com.arm.armcloudsdk.bean.RequestOptions
import com.arm.armcloudsdk.bean.SystemKeyStrokeEnum
import com.arm.armcloudsdk.bean.VideoDefinitionEnum
import com.arm.armcloudsdk.config.PhonePlayConfig
import com.arm.armcloudsdk.def.EngineState
import com.arm.armcloudsdk.def.LocationMode
import com.arm.armcloudsdk.def.ScreenRotationType
import com.arm.armcloudsdk.def.ScreenShotType
import com.arm.armcloudsdk.def.StreamType
import com.arm.armcloudsdk.def.UploadFilePath
import com.arm.armcloudsdk.def.VideoRenderMode
import com.arm.armcloudsdk.def.VideoRotationMode
import com.arm.armcloudsdk.dto.RoomInfoDto
import com.arm.armcloudsdk.innerapi.IClipBoardListener
import com.arm.armcloudsdk.innerapi.IMessageChannel
import com.arm.armcloudsdk.innerapi.IPlayerListener
import com.arm.armcloudsdk.innerapi.IStreamListener
import com.arm.armcloudsdk.innerapi.IUploadFileCallBack
import com.arm.armcloudsdk.innerapi.LocationEventListener
import com.arm.armcloudsdk.innerapi.ScreenShotCallBack
import com.arm.armcloudsdk.innerapi.SetAutoRecycleTimeCallback
import com.arm.armcloudsdk.innerapi.StreamProfileChangeCallBack
import com.armcloud.armcloudsdkdemo.dialog.MultiPadCodesPop
import com.armcloud.armcloudsdkdemo.dialog.RomCodePop
import com.armcloud.armcloudsdkdemo.dtos.MultiPadCodeDto
import com.blankj.utilcode.util.GsonUtils
import com.blankj.utilcode.util.SPUtils
import com.blankj.utilcode.util.ToastUtils
import com.blankj.utilcode.util.UriUtils
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import kotlinx.coroutines.withTimeout
import java.io.BufferedReader
import java.io.InputStreamReader
import java.io.OutputStream
import java.lang.reflect.Type
import kotlin.concurrent.thread


class MainActivity : AppCompatActivity() {


    companion object {
        private const val TAG = "MainActivity"
        private const val KEY_ROOM_CODE = "romCodeKey"
    }

    private var isAuto = false
    private var tvUseLocal: TextView? = null
    private var tvUseGyroscope: TextView? = null
    private var tvUserAccelerometer: TextView? = null
    private var tvEnableLocation: TextView? = null
    private var tvVibrator: TextView? = null
    private var tvCamera: TextView? = null
    private var tvMic: TextView? = null
    private var tvCloudSync: TextView? = null
    private var tvLocalSync: TextView? = null
    private var tvRoomId: AppCompatTextView? = null
    private var tvQuality: AppCompatTextView? = null
    private var mRomCodePop: RomCodePop? = null
    private var mMultiPadCodesPop: MultiPadCodesPop? = null
    private var uploadPodCode = ""
    private var uploadToken = ""
    private var uploadPath = UploadFilePath.PATH_DOWNLOAD
    private var multiPadCodes = mutableListOf<String>()
    private var mChosePadCodes = mutableListOf<String>()
    private var tvMultiPadCodes: AppCompatTextView? = null
    private var isMultiControl = false

    // 配置
    private var enableLocalKeyboard = false
        set(value) {
            tvUseLocal?.text = "使用手机本地输入法: $value"
            field = value
        }
    private var enableGyroscopeSensor = false
        set(value) {
            tvUseGyroscope?.text = "启用陀螺仪: $value"
            field = value
        }
    private var enableAccelerometer = false
        set(value) {
            tvUserAccelerometer?.text = "启用重力传感器: $value"
            field = value
        }
    private var enableLocationService = false
        set(value) {
            tvEnableLocation?.text = "能否使用定位: $value"
            field = value
        }
    private var enableCamera = true
        set(value) {
            tvCamera?.text = "开启摄像头权限: $value"
            field = value
        }
    private var enableMic = false
        set(value) {
            tvMic?.text = "开启麦克风权限: $value"
            field = value
        }
    private var enableCloudSync = true
        set(value) {
            tvCloudSync?.text = "云机剪切板同步真机: $value"
            field = value
        }
    private var enableLocalSync = true
        set(value) {
            tvLocalSync?.text = "真机剪切板同步云机: $value"
            field = value
        }
    private var enableVibrator = false
        set(value) {
            tvVibrator?.text = "能否使用震动: $value"
            field = value
        }

    private var etPodId: EditText? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        multiPadCodes = readJsonFromAssets(applicationContext, "pad_codes") ?: mutableListOf()

        mRomCodePop = RomCodePop(this)
        mMultiPadCodesPop = MultiPadCodesPop(this)
        mMultiPadCodesPop?.updateList(multiPadCodes.map {
            MultiPadCodeDto(it)
        }.toMutableList())

        initRomCode()

        tvRoomId = findViewById(R.id.tv_room_id)
        val fl = findViewById<FrameLayout>(R.id.fl_content)
        val dl = findViewById<DrawerLayout>(R.id.dl_container)
        tvQuality = findViewById(R.id.tv_network_quality)
        // 打开操作侧边栏
        findViewById<TextView>(R.id.tv_start_left).setOnClickListener {
            dl.openDrawer(Gravity.LEFT)
        }
        // 启动
        findViewById<TextView>(R.id.create_room_btn).setOnClickListener {
            Result.runCatching {
                ArmCloudEngine.stop()
                toStart(fl)
            }.onFailure {
                Toast.makeText(this@MainActivity, "error: ${it.message}", Toast.LENGTH_SHORT).show()
            }
        }
        findViewById<TextView>(R.id.tv_stop).setOnClickListener {
            ArmCloudEngine.stop()
            tvRoomId?.text = ""
            tvQuality?.text = ""
        }
        val mute = findViewById<TextView>(R.id.mute_btn)
        mute.setOnClickListener {
            ArmCloudEngine.muteAudio(true)
        }
        val unMute = findViewById<TextView>(R.id.unmute_btn)
        unMute.setOnClickListener {
            ArmCloudEngine.muteAudio(false)
        }
        val screenshot = findViewById<TextView>(R.id.screenshot_btn)
        screenshot.setOnClickListener {
            showScreenShotDialog(this)
        }
        findViewById<TextView>(R.id.btn_custom_definition).setOnClickListener {
            toSetDefinition()
        }
        findViewById<TextView>(R.id.btn_low).setOnClickListener {
            ArmCloudEngine.videoStreamProfileId(VideoDefinitionEnum.DEFINITION_L)
        }
        findViewById<TextView>(R.id.btn_height).setOnClickListener {
            ArmCloudEngine.videoStreamProfileId(VideoDefinitionEnum.DEFINITION_S)
        }
        findViewById<TextView>(R.id.btn_transparent_transmission).setOnClickListener {
            ArmCloudEngine.getMessageChannel()
                ?.sendMessage(1, "app透传消息", "aidl.cloud.api.MessageServer", "com.cloud.control")
        }
        val tv = findViewById<TextView>(R.id.btn_auto_portrait)
        tv.setOnClickListener {
            isAuto = !isAuto
            val mode = if (isAuto) VideoRotationMode.INTERNAL else VideoRotationMode.EXTERNAL
            ArmCloudEngine.setVideoRotationMode(mode)
            tv.text = "自适应横竖屏: $isAuto"
        }
        findViewById<TextView>(R.id.btn_active_landscape).setOnClickListener {
            ArmCloudEngine.rotate(ScreenRotationType.TYPE_LANDSCAPE)
            isAuto = false
            tv.text = "自适应横竖屏: $isAuto"
        }
        findViewById<TextView>(R.id.btn_active_portrait).setOnClickListener {
            ArmCloudEngine.rotate(ScreenRotationType.TYPE_PORTRAIT)
            isAuto = false
            tv.text = "自适应横竖屏: $isAuto"
        }
        val tvLocation = findViewById<TextView>(R.id.btn_change_location_type)
        tvLocation.setOnClickListener {
            ArmCloudEngine.getLocationServer()?.enableLocationService(true)
            ArmCloudEngine.setLocationServiceMode(LocationMode.MODE_AUTO)
            Toast.makeText(this, "当前使用自动定位", Toast.LENGTH_SHORT).show()
        }
        tvUseLocal = findViewById<TextView>(R.id.btn_use_local)
        tvUseLocal?.setOnClickListener {
            enableLocalKeyboard = !enableLocalKeyboard
            tvUseLocal?.text = "使用手机本地输入法: $enableLocalKeyboard"
            ArmCloudEngine.enableLocalKeyboard(enableLocalKeyboard)
        }

        tvUseGyroscope = findViewById<TextView>(R.id.btn_use_gyroscope)
        tvUseGyroscope?.setOnClickListener {
            enableGyroscopeSensor = !enableGyroscopeSensor
            tvUseGyroscope?.text = "启用陀螺仪: $enableGyroscopeSensor"
            ArmCloudEngine.enableGyroscopeSensor(enableGyroscopeSensor)
        }
        tvUserAccelerometer = findViewById<TextView>(R.id.btn_use_accelerometer)
        tvUserAccelerometer?.setOnClickListener {
            enableAccelerometer = !enableAccelerometer
            tvUserAccelerometer?.text = "启用重力传感器: $enableAccelerometer"
            ArmCloudEngine.enableAccelerometerSensor(enableAccelerometer)
        }

        tvEnableLocation = findViewById<TextView>(R.id.btn_enable_location)
        tvEnableLocation?.setOnClickListener {
            enableLocationService = !enableLocationService
            tvEnableLocation?.text = "能否使用定位: $enableLocationService"
            ArmCloudEngine.getLocationServer()?.enableLocationService(enableLocationService)
        }
        tvVibrator = findViewById<TextView>(R.id.btn_enable_vibrator)
        tvVibrator?.setOnClickListener {
            enableVibrator = !enableVibrator
            tvVibrator?.text = "能否使用震动: $enableVibrator"
            ArmCloudEngine.enableVibrator(enableVibrator)
        }

        tvCamera = findViewById<TextView>(R.id.btn_enable_camera)
        tvCamera?.setOnClickListener {
            enableCamera = !enableCamera
            tvCamera?.text = "开启摄像头权限: $enableCamera"
            ArmCloudEngine.enableCamera(enableCamera)
        }

        tvMic = findViewById<TextView>(R.id.btn_enable_mic)
        tvMic?.setOnClickListener {
            enableMic = !enableMic
            tvMic?.text = "开启麦克风权限: $enableMic"
            ArmCloudEngine.enableMic(enableMic)
        }

        tvCloudSync = findViewById<TextView>(R.id.btn_enable_cloud_sync)
        tvCloudSync?.setOnClickListener {
            enableCloudSync = !enableCloudSync
            tvCloudSync?.text = "云机剪切板同步真机: $enableCloudSync"
            ArmCloudEngine.getClipBoardServiceManager()
                ?.enableClipboardCloudPhoneSync(enableCloudSync)
        }

        tvLocalSync = findViewById<TextView>(R.id.btn_enable_local_sync)
        tvLocalSync?.setOnClickListener {
            enableLocalSync = !enableLocalSync
            tvLocalSync?.text = "真机剪切板同步云机: $enableLocalSync"
            ArmCloudEngine.getClipBoardServiceManager()
                ?.enableClipboardLocalPhoneSync(enableLocalSync)
        }

        findViewById<TextView>(R.id.btn_send_keyboard).setOnClickListener {
            showSendKeyBoardDialog(this)
        }

        findViewById<TextView>(R.id.btn_send_location).setOnClickListener {
            showLocationDialog(this, object : OnThreeInputDialogListener {
                override fun onPositiveClick(
                    longitude: String,
                    latitude: String,
                    bitrateId: String
                ) {
                    Result.runCatching {
                        ArmCloudEngine.getLocationServer()?.enableLocationService(true)
                        ArmCloudEngine.setLocationServiceMode(LocationMode.MODE_MANUAL)
                        ArmCloudEngine.getLocationServer()?.remoteLocationMock(
                            latitude.toDouble(),
                            longitude.toDouble()
                        )
                    }
                }

                override fun onNegativeClick() {
                }
            })
        }

        findViewById<TextView>(R.id.btn_change_render_mode).setOnClickListener {
            showChangeModeDialog(this)
        }

        findViewById<TextView>(R.id.btn_auto_recycler_time).setOnClickListener {
            Result.runCatching {
                showAutoRecyclerTimeDialog(this@MainActivity)
            }.onFailure {
                Toast.makeText(this@MainActivity, "error: ${it.message}", Toast.LENGTH_SHORT).show()
            }
        }

        findViewById<TextView>(R.id.btn_get_auto_recycler_time).setOnClickListener {
            val autoRecycleTime = ArmCloudEngine.getAutoRecycleTime()
            Toast.makeText(
                this@MainActivity,
                "当前 无操作超时时间： $autoRecycleTime s",
                Toast.LENGTH_SHORT
            ).show()
        }

        findViewById<AppCompatTextView>(R.id.tv_back).setOnClickListener {
            ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.BACK)
        }
        findViewById<AppCompatTextView>(R.id.tv_home).setOnClickListener {
            ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.HOME)
        }
        findViewById<AppCompatTextView>(R.id.tv_job).setOnClickListener {
            ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.JOB)
        }
        findViewById<AppCompatTextView>(R.id.tv_send_input_txt).setOnClickListener {
            // 获取剪贴板管理器
            val clipboardManager = getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
            // 检查剪贴板中是否有内容
            if (!clipboardManager.hasPrimaryClip()) {
                return@setOnClickListener
            }
            val clipData = clipboardManager.primaryClip
            val item = clipData!!.getItemAt(0)
            val text = item.text.toString()
            ArmCloudEngine.sendInputText(text)
        }
        findViewById<TextView>(R.id.btn_upload_file).setOnClickListener {
            toUploadFile()
        }
        findViewById<TextView>(R.id.btn_clipboard_message).setOnClickListener {
            ArmCloudEngine.getClipBoardServiceManager()?.sendClipBoardMessage("123")
        }

        findViewById<TextView>(R.id.btn_keyboard_message).setOnClickListener {
            ArmCloudEngine.getLocalInputManager()?.sendInputText("123")
        }
        setListener()
    }

    private fun readJsonFromAssets(context: Context, fileName: String): MutableList<String>? {
        val assetManager = context.assets
        return try {
            // 打开文件
            val inputStream = assetManager.open(fileName)
            val reader = BufferedReader(InputStreamReader(inputStream))
            // 读取文件内容
            val stringBuilder = StringBuilder()
            var line: String?
            while (reader.readLine().also { line = it } != null) {
                stringBuilder.append(line)
            }
            reader.close()
            // 使用 Gson 将 JSON 解析为 List
            val gson = Gson()
            gson.fromJson(stringBuilder.toString(), object : TypeToken<List<String?>?>() {}.type)
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            null
        }
    }

    private fun toUploadFile() {
        val builder: AlertDialog.Builder = AlertDialog.Builder(this)
        val inflater = LayoutInflater.from(this)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_upload_file_dialog, null)
        builder.setView(dialogView)
        val etPodCode = dialogView.findViewById<EditText>(R.id.et_pod_code)
        val etToken = dialogView.findViewById<EditText>(R.id.et_token)
        val rbNone = dialogView.findViewById<RadioButton>(R.id.rb_none)
        val rbDcim = dialogView.findViewById<RadioButton>(R.id.rb_dcim)
        val rbDocuments = dialogView.findViewById<RadioButton>(R.id.rb_documents)
        val rbDownload = dialogView.findViewById<RadioButton>(R.id.rb_download)
        val rbMovies = dialogView.findViewById<RadioButton>(R.id.rb_movies)
        val rbMusic = dialogView.findViewById<RadioButton>(R.id.rb_music)
        val rbPictures = dialogView.findViewById<RadioButton>(R.id.rb_pictures)
        builder.setPositiveButton("确定") { dialog, which ->
            uploadPodCode = etPodCode.text.toString()
            uploadToken = etToken.text.toString()
            if (uploadPodCode.isBlank()) {
                ToastUtils.showShort("请输入PodCode")
                return@setPositiveButton
            }
            if (uploadToken.isBlank()) {
                ToastUtils.showShort("请输入Token")
                return@setPositiveButton
            }
            uploadPath = ""
            if (rbNone.isChecked) {
                uploadPath = ""
            }
            if (rbDcim.isChecked) {
                uploadPath = UploadFilePath.PATH_DCIM
            }
            if (rbDocuments.isChecked) {
                uploadPath = UploadFilePath.PATH_DOCUMENTS
            }
            if (rbDownload.isChecked) {
                uploadPath = UploadFilePath.PATH_DOWNLOAD
            }
            if (rbMovies.isChecked) {
                uploadPath = UploadFilePath.PATH_MOVIES
            }
            if (rbMusic.isChecked) {
                uploadPath = UploadFilePath.PATH_MUSIC
            }
            if (rbPictures.isChecked) {
                uploadPath = UploadFilePath.PATH_PICTURES
            }
            intoFileManager()
        }
        builder.setNegativeButton("取消") { dialog, which -> }
        builder.show()
    }

    private fun intoFileManager() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.setType("*/*") //无类型限制
        //        有类型限制是这样的:
//        intent.setType(“image/*”);//选择图片
//        intent.setType(“audio/*”); //选择音频
//        intent.setType(“video/*”); //选择视频 （mp4 3gp 是android支持的视频格式）
//        intent.setType(“video/*;image/*”);//同时选择视频和图片
        intent.addCategory(Intent.CATEGORY_OPENABLE)
        startActivityForResult(intent, 1)
    }

    private fun initRomCode() {
        val roomCodeStr = SPUtils.getInstance().getString(KEY_ROOM_CODE)
        if (roomCodeStr.isNullOrBlank()) {
            return
        }
        val listType: Type = object : TypeToken<List<String>>() {}.type
        val romCodeList = GsonUtils.fromJson<MutableList<String>>(roomCodeStr, listType)
        mRomCodePop?.updateList(romCodeList)
    }

    private fun setListener() {
        mRomCodePop?.setOnItemCLickListener(object : RomCodePop.OnItemCLickListener {
            override fun onItemCLick(str: String) {
                etPodId?.setText(str)
                etPodId?.setSelection(etPodId?.text?.length ?: 0)
            }
        })

        mMultiPadCodesPop?.setOnItemCLickListener(object : MultiPadCodesPop.OnItemCLickListener {
            override fun onItemCLick(multiPadCodeDtoList: List<String>) {
                mChosePadCodes = multiPadCodeDtoList.toMutableList()
                Log.e("am", "mChosePadCodes ===> $mChosePadCodes")
                tvMultiPadCodes?.text = mChosePadCodes.joinToString("，")
            }
        })
    }

    private fun showScreenShotDialog(context: MainActivity) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.screen_shot_dialog, null)
        builder.setView(dialogView)
        val rbLocal = dialogView.findViewById<RadioButton>(R.id.rb_local)
        val rbCloud = dialogView.findViewById<RadioButton>(R.id.rb_cloud)
        builder.setPositiveButton("确定") { dialog, which ->
            if (rbLocal.isChecked) {
                ArmCloudEngine.screenShot(ScreenShotType.TYPE_LOCAL)
            }
            if (rbCloud.isChecked) {
                ArmCloudEngine.screenShot(ScreenShotType.TYPE_CLOUD)
            }
        }
        builder.setNegativeButton("取消") { dialog, which -> }
        builder.show()
    }

    private fun showAutoRecyclerTimeDialog(context: MainActivity) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.auto_recycler_time_dialog, null)
        builder.setView(dialogView)
        val etAutoRecycler = dialogView.findViewById<EditText>(R.id.et_auto_refcy_time)
        builder.setPositiveButton("确定") { dialog, which ->
            var autoRecyclerTime = "300"
            if (etAutoRecycler.text.isNotBlank()) {
                autoRecyclerTime = etAutoRecycler.text.toString().trim()
                if (autoRecyclerTime.toInt() <= 0 || autoRecyclerTime.toInt() > 7200) {
                    Toast.makeText(
                        this,
                        "自动无操作回收时间设置出错  范围: 1 ~ 7200",
                        Toast.LENGTH_SHORT
                    ).show()
                    return@setPositiveButton
                }
            }
            ArmCloudEngine.autoRecycleTime(autoRecyclerTime.toInt())
        }
        builder.setNegativeButton("取消") { dialog, which -> }
        builder.show()
    }

    private fun showChangeModeDialog(context: MainActivity) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_render_mode_dialog, null)
        builder.setView(dialogView)
        val rbAuto = dialogView.findViewById<RadioButton>(R.id.rb_auto)
        val rbFill = dialogView.findViewById<RadioButton>(R.id.rb_fill)
        builder.setPositiveButton("确定") { dialog, which ->
            if (rbAuto.isChecked) {
                ArmCloudEngine.updateVideoRenderMode(VideoRenderMode.VIDEO_RENDER_MODE_FIT)
            }
            if (rbFill.isChecked) {
                ArmCloudEngine.updateVideoRenderMode(VideoRenderMode.VIDEO_RENDER_MODE_FILL)
            }
        }
        builder.setNegativeButton("取消") { dialog, which -> }
        builder.show()
    }

    private fun showSendKeyBoardDialog(context: MainActivity) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_send_keyboard_dialog, null)
        builder.setView(dialogView)

        val radioGroup = dialogView.findViewById<RadioGroup>(R.id.rg)
        builder.setPositiveButton("确定") { dialog, which ->
            when (radioGroup.checkedRadioButtonId) {
                R.id.rb_back -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.BACK)
                }

                R.id.rb_home -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.HOME)
                }

                R.id.rb_job -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.JOB)
                }

                R.id.rb_menu -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.MENU)
                }

                R.id.rb_volume_add -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.VOLUME_ADD)
                }

                R.id.rb_volume_less -> {
                    ArmCloudEngine.sendKeyEvent(SystemKeyStrokeEnum.VOLUME_LESS)
                }
            }
        }
        builder.setNegativeButton("取消") { dialog, which -> }
        builder.show()
    }

    private fun toSetDefinition() {
        showCustomDefinitionDialog(this@MainActivity, object : OnThreeInputDialogListener {
            override fun onPositiveClick(
                resolutionId: String,
                frameId: String,
                bitrateId: String
            ) {
                ArmCloudEngine.videoStreamProfileId(
                    resolutionId.toInt(),
                    frameId.toInt(),
                    bitrateId.toInt()
                )
            }

            override fun onNegativeClick() {
            }
        })
    }

    private fun toStart(fl: ViewGroup) {
        showTwoInputDialog(this@MainActivity, object : OnTwoInputDialogListener {
            override fun onPositiveClick(
                userId: String?,
                token: String?,
                podId: String,
                clientType: String,
                ve: VideoDefinitionEnum,
                s: Int,
                sType: String,
                renderMode: Int,
                rotation: Int,
                autoRecyclerTime: String
            ) {
                if (userId == null || token == null) {
                    return
                }
                Result.runCatching {
                    val romCodeList = mRomCodePop?.getList() ?: mutableListOf()
                    romCodeList.add(0, podId)
                    val distinctList = romCodeList.distinct().toMutableList()
                    mRomCodePop?.updateList(distinctList)
                    SPUtils.getInstance().put(KEY_ROOM_CODE, GsonUtils.toJson(distinctList))
                    val dto = RoomInfoDto(
                        userId,
                        token,
                        podId,
                        clientType
                    )
                    start(fl, dto, ve, s, sType, renderMode, rotation, autoRecyclerTime)
                }.onFailure {
                    Log.e("am", "failure ===> ${it.message}")
                    Toast.makeText(
                        this@MainActivity,
                        "创建房间失败 msg: ${it.message}",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onNegativeClick() {
            }
        })
    }

    private fun start(
        container: ViewGroup,
        dto: RoomInfoDto,
        videoStreamProfileId: VideoDefinitionEnum,
        rotation: Int,
        streamType: String,
        renderMode: Int,
        videoRotationMode: Int,
        autoRecyclerTime: String
    ) {
        val builder: PhonePlayConfig.Builder = PhonePlayConfig.Builder()
        builder.context(this)
            .userId(dto.userId) //必填参数 自定义客户端用户 ID
            .padCode(dto.padCode) // 必填参数, 云手机实例 ID
            .token(dto.token) // 必填参数，临时鉴权 token
            .clientType(dto.clientType) //必填参数 客户端类型
            .container(container) // 必填参数，用来承载画面的 Container, 参数说明: layout 需要是 FrameLayout 或者 FrameLayout 的子类
            .enableMultiControl(isMultiControl) // 选填参数 是否开启群控
            .setPadCodes(mChosePadCodes) // 选填参数 群控设备号集合
            .rotation(rotation) // 选填参数 屏幕的横竖屏 默认竖屏
            .videoStreamProfileId(
                videoStreamProfileId.definition,
                videoStreamProfileId.frameRate,
                videoStreamProfileId.bitrate
            ) // 选填参数，清晰度档位ID 默认高清
            .enableGyroscopeSensor(enableGyroscopeSensor) // 选填参数 打开陀螺仪开关 默认false
            .enableVibrator(enableVibrator) // 选填参数 打开本地振动开关 默认false
            .enableLocationService(enableLocationService) // 选填参数 打开本地定位功能开关 默认false
            .enableLocalKeyboard(enableLocalKeyboard) // 选填参数 打开本地键盘开关 默认false
            .enableClipboardCloudPhoneSync(enableCloudSync) // 选填参数 打开云机剪切板同步至真机 默认true
            .enableClipboardLocalPhoneSync(enableLocalSync) // 选填参数 打开真机剪切板同步至云机 默认true
            .enableCamera(enableCamera) // 选填参数 开启相机权限 默认 true
            .enableMic(enableMic) // 选填参数 开启麦克风权限 默认 false
            .streamType(streamType) // 选填参数 指定启动云手机时拉取音视频流类型 默认拉取音视频流
            .videoRenderMode(renderMode) // 选填参数 指定视频流渲染模式 默认等比缩放居中模式
            .videoRotationMode(videoRotationMode) // 选填参数 指定视频旋转模式 默认非SDK处理旋转
            .autoRecycleTime(autoRecyclerTime.toInt())
            // 获取音视频流信息回调监听
            .streamListener(object : IStreamListener {
                override fun onFirstAudioFrame(uid: String) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "订阅视频流到收到音频首帧回调",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }

                override fun onFirstRemoteVideoFrame(uid: String, width: Int, height: Int) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "订阅视频流到收到视频首帧回调",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }

                override fun onVideoSizeChanged(uid: String, width: Int, height: Int) {
                }

                override fun onStreamPaused() {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "调用 pause()，暂停播放后的回调",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }

                override fun onStreamConnectionStateChanged(state: Int, reason: Int) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "视频流连接状态变更回调 state: $state",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }

                override fun onScreenRotation(rotationType: Int) {
                    runOnUiThread {
                        requestedOrientation =
                            if (rotationType == ScreenRotationType.TYPE_PORTRAIT) ActivityInfo.SCREEN_ORIENTATION_PORTRAIT else ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE
                        Toast.makeText(
                            this@MainActivity,
                            "屏幕旋转：${if (rotationType == ScreenRotationType.TYPE_PORTRAIT) "竖屏" else "横屏"}",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }
            })
        lifecycleScope.launch(Dispatchers.IO) {
            withTimeout(5000) {
                while (ArmCloudEngine.engineState != EngineState.STATE_AVAILABLE) {
                    delay(200)
                }
                withContext(Dispatchers.Main) {
                    toDelayStart(builder)
                }
            }
        }
    }

    private fun toDelayStart(builder: PhonePlayConfig.Builder) {
        ArmCloudEngine.start(builder.build(), object : IPlayerListener {
            override fun onPlaySuccess(videoStreamProfileId: Int) {
                Log.e("am", "onPlaySuccess ===> videoStreamProfileId: $videoStreamProfileId")
                runOnUiThread {
                    tvRoomId?.text = "当前房间： ${builder.padCode}"
                    Toast.makeText(this@MainActivity, "拉流成功", Toast.LENGTH_SHORT).show()
                }
                initListener()
            }

            override fun onError(code: Int, msg: String) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "onError code：$code msg: $msg ",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onWarning(code: Int, msg: String) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "onWarning code：$code  $msg ",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onNetworkChanged(var1: Int) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "onNetworkChanged code：$var1",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onServiceInit(extras: Map<String, Any>?) {
                runOnUiThread {
                    ArmCloudEngine.getClipBoardServiceManager()?.setBoardSyncClipListener(object :
                        IClipBoardListener {
                        override fun onClipBoardMessageReceived(data: String) {
                            Toast.makeText(
                                this@MainActivity,
                                "收到剪切板消息： $data",
                                Toast.LENGTH_SHORT
                            ).show()
                        }
                    })
                    Toast.makeText(this@MainActivity, "onServiceInit", Toast.LENGTH_SHORT).show()
                }
            }

            override fun networkQualityRtt(rtt: Int) {
                runOnUiThread {
                    tvQuality?.text = "NetworkQuality: $rtt ms"
//                    Toast.makeText(this@MainActivity, "networkQualityRtt $rtt", Toast.LENGTH_SHORT).show()
                }
            }

            override fun onMultiCloudPhoneJoin(padCode: String) {
                runOnUiThread {
                    ToastUtils.showShort("群控用户加入")
                }
            }

            override fun onMultiCloudPhoneLeave(padCode: String) {
                runOnUiThread {
                    ToastUtils.showShort("群控用户离开")
                }
            }
        })
    }

    private fun initListener() {
        ArmCloudEngine.setStreamProfileChangeListener(object : StreamProfileChangeCallBack {
            override fun onVideoStreamProfileChange(
                isSuccess: Boolean,
                from: String,
                current: String
            ) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "清晰度切换成功 from：$from  to: $current",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        })
        ArmCloudEngine.setAutoRecycleTimeCallback(object : SetAutoRecycleTimeCallback {
            override fun onResult(autoRecycleTime: Int) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "无操作回收服务回调 超时时间：$autoRecycleTime",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        })
        ArmCloudEngine.setScreenShotCallBack(object : ScreenShotCallBack {
            override fun onScreenShot(bitmap: Bitmap) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "本地获取到截图：${bitmap.byteCount}",
                        Toast.LENGTH_SHORT
                    ).show()
                    saveImageToGallery(this@MainActivity, bitmap, "shotScreen", "shotScreen")
                }
            }
        })
        ArmCloudEngine.setLocationEventListener(object : LocationEventListener {
            override fun onReceivedRemoteLocationRequest(requestOptions: RequestOptions) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "云机请求开启定位",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onRemoteLocationRequestEnded() {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "云机请求停止定位",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onSentLocalLocation(locationInfo: LocationInfo) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "本机发送定位",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

            override fun onRemoteLocationUpdated(locationInfo: LocationInfo) {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        "云机位置更新",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        })
        ArmCloudEngine.getMessageChannel()
            ?.setMessageListener(object : IMessageChannel.IMessageReceiver {
                override fun onReceiveMessage(message: String) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "接收到消息：$message",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }
            })
        ArmCloudEngine.getMessageChannel()
            ?.setMessageListener(object : IMessageChannel.IMessageReceiver {
                override fun onReceiveMessage(message: String) {
                    val clipboardManager: ClipboardManager =
                        getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                    // 创建一个新的 ClipData，将文本放入
                    val clipData = ClipData.newPlainText("label", message)
                    // 将 ClipData 设置为剪贴板的主剪贴板内容
                    clipboardManager.setPrimaryClip(clipData)
                }
            })
    }

    interface OnThreeInputDialogListener {
        fun onPositiveClick(
            resolutionId: String,
            frameId: String,
            bitrateId: String
        )

        fun onNegativeClick()
    }

    interface OnTwoInputDialogListener {
        fun onPositiveClick(
            userId: String?,
            token: String?,
            podId: String,
            clientType: String,
            ve: VideoDefinitionEnum,
            s: Int,
            sType: String,
            renderMode: Int,
            rotation: Int,
            autoRecyclerTime: String
        )

        fun onNegativeClick()
    }

    private fun showCustomDefinitionDialog(
        context: Context?,
        listener: OnThreeInputDialogListener
    ) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_three_input_dialog, null)
        builder.setView(dialogView)
        val resolution = dialogView.findViewById<EditText>(R.id.et_resolution)
        val rate = dialogView.findViewById<EditText>(R.id.et_frame_rate)
        val bitrate = dialogView.findViewById<EditText>(R.id.et_bitrate)

        builder.setPositiveButton("确定") { dialog, which ->

            Result.runCatching {
                val resolutionStr = resolution.text.toString().toInt()
                val rateStr = rate.text.toString().toInt()
                val bitrateStr = bitrate.text.toString().toInt()

                if (resolutionStr !in 7..20) {
                    Toast.makeText(
                        this@MainActivity,
                        "清晰度设置出错 范围 7~20",
                        Toast.LENGTH_SHORT
                    ).show()
                    return@setPositiveButton
                }
                if (rateStr !in 1..9) {
                    Toast.makeText(this@MainActivity, "帧率设置出错 范围 1~9", Toast.LENGTH_SHORT)
                        .show()
                    return@setPositiveButton
                }
                if (bitrateStr !in 1..15) {
                    Toast.makeText(this@MainActivity, "码率设置出错 范围 1~15", Toast.LENGTH_SHORT)
                        .show()
                    return@setPositiveButton
                }

                listener.onPositiveClick(
                    resolutionStr.toString(),
                    rateStr.toString(),
                    bitrateStr.toString()
                )
            }.onFailure {
                Toast.makeText(this, "请输入数值", Toast.LENGTH_SHORT).show()
            }
        }

        builder.setNegativeButton("取消") { dialog, which -> listener.onNegativeClick() }
        builder.show()
    }


    private fun showLocationDialog(
        context: Context?,
        listener: OnThreeInputDialogListener
    ) {
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_location_dialog, null)
        builder.setView(dialogView)
        val longitude = dialogView.findViewById<EditText>(R.id.et_longitude)
        val latitude = dialogView.findViewById<EditText>(R.id.et_latitude)

        builder.setPositiveButton("确定") { dialog, which ->
            Result.runCatching {
                val longitudeStr = longitude.text.toString().toFloat()
                val latitudeStr = latitude.text.toString().toFloat()
                if (longitudeStr < -180 || longitudeStr > 180) {
                    Toast.makeText(this@MainActivity, "经度不在范围内", Toast.LENGTH_SHORT).show()
                    return@setPositiveButton
                }
                if (latitudeStr < -90 || latitudeStr > 90) {
                    Toast.makeText(this@MainActivity, "纬度不在范围内", Toast.LENGTH_SHORT).show()
                    return@setPositiveButton
                }
                listener.onPositiveClick(
                    longitudeStr.toString(),
                    latitudeStr.toString(),
                    ""
                )
            }.onFailure {
                Toast.makeText(this, "请输入数值", Toast.LENGTH_SHORT).show()
            }
        }
        builder.setNegativeButton("取消") { dialog, which -> listener.onNegativeClick() }
        builder.show()
    }

    private fun showTwoInputDialog(context: Context?, listener: OnTwoInputDialogListener) {
        isMultiControl = false
        mChosePadCodes.clear()
        val builder: AlertDialog.Builder = AlertDialog.Builder(context)
        val inflater = LayoutInflater.from(context)
        // 自定义弹窗布局
        val dialogView: View = inflater.inflate(R.layout.custom_two_input_dialog, null)
        builder.setView(dialogView)
        val userId = dialogView.findViewById<EditText>(R.id.et_userid)
        val token = dialogView.findViewById<EditText>(R.id.et_token)
        etPodId = dialogView.findViewById(R.id.et_podid)
        if (!mRomCodePop?.getList().isNullOrEmpty()) {
            val msg = mRomCodePop!!.getList()!![0]
            etPodId?.setText(msg)
        }
        val clientType = dialogView.findViewById<EditText>(R.id.et_client_type)

        // 群控
        val llMulti = dialogView.findViewById<LinearLayoutCompat>(R.id.ll_multi)
        dialogView.findViewById<AppCompatTextView>(R.id.tv_history).setOnClickListener {
            mRomCodePop?.show()
        }
        tvMultiPadCodes = dialogView.findViewById<AppCompatTextView>(R.id.tv_multi_pad_codes)
        val btnMultiChose = dialogView.findViewById<AppCompatButton>(R.id.btn_chose_multi_pad_codes)

        val radioGroup = dialogView.findViewById<RadioGroup>(R.id.rg_control_model)
        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            Log.e("am", "checkedId ===> $checkedId")
            when(checkedId) {
                R.id.rb_single -> {
                    isMultiControl = false
                    llMulti.visibility = View.GONE
                }
                R.id.rb_multi -> {
                    isMultiControl = true
                    llMulti.visibility = View.VISIBLE
                }
            }
        }

        btnMultiChose.setOnClickListener {
            mMultiPadCodesPop?.show()
        }

        builder.setPositiveButton("确定") { dialog, which ->
            val sensor = dialogView.findViewById<CheckBox>(R.id.rb_use_gyroscope_sensor)
            val keyboard = dialogView.findViewById<CheckBox>(R.id.rb_use_local_keyboard)
            val location = dialogView.findViewById<CheckBox>(R.id.rb_use_location)
            val vibrator = dialogView.findViewById<CheckBox>(R.id.rb_use_vibrator)
            val camera = dialogView.findViewById<CheckBox>(R.id.rb_use_camera)
            val mic = dialogView.findViewById<CheckBox>(R.id.rb_use_mic)
            val cloudSync = dialogView.findViewById<CheckBox>(R.id.rb_use_cloud_sync)
            val localSync = dialogView.findViewById<CheckBox>(R.id.rb_use_local_sync)

            enableGyroscopeSensor = sensor.isChecked
            enableLocalKeyboard = keyboard.isChecked
            enableLocationService = location.isChecked
            enableVibrator = vibrator.isChecked
            enableCamera = camera.isChecked
            enableMic = mic.isChecked
            enableCloudSync = cloudSync.isChecked
            enableLocalSync = localSync.isChecked

            val rg = dialogView.findViewById<RadioGroup>(R.id.rg)
            val rbLow = rg.findViewById<RadioButton>(R.id.rb_low)
            val rbH = rg.findViewById<RadioButton>(R.id.rb_h)
            val rbS = rg.findViewById<RadioButton>(R.id.rb_s)

            var renderMode = VideoRenderMode.VIDEO_RENDER_MODE_FIT
            val rbAuto = dialogView.findViewById<RadioButton>(R.id.rb_auto)
            val rbFill = dialogView.findViewById<RadioButton>(R.id.rb_fill)
            if (rbAuto.isChecked) {
                renderMode = VideoRenderMode.VIDEO_RENDER_MODE_FIT
            }
            if (rbFill.isChecked) {
                renderMode = VideoRenderMode.VIDEO_RENDER_MODE_FILL
            }

            var rotation = VideoRotationMode.EXTERNAL
            val rBExternal = dialogView.findViewById<RadioButton>(R.id.rb_external)
            val rBInternal = dialogView.findViewById<RadioButton>(R.id.rb_internal)
            if (rBExternal.isChecked) {
                rotation = VideoRotationMode.EXTERNAL
            }
            if (rBInternal.isChecked) {
                rotation = VideoRotationMode.INTERNAL
            }

            val ve: VideoDefinitionEnum = if (rbLow.isChecked) {
                VideoDefinitionEnum.DEFINITION_L
            } else if (rbH.isChecked) {
                VideoDefinitionEnum.DEFINITION_H
            } else {
                VideoDefinitionEnum.DEFINITION_S
            }

            val rgs = dialogView.findViewById<RadioGroup>(R.id.rg_s)
            val landscape = rgs.findViewById<RadioButton>(R.id.rb_landscape)
            val portrait = rgs.findViewById<RadioButton>(R.id.rb_portrait)
            var s = ScreenRotationType.TYPE_PORTRAIT
            if (landscape.isChecked) {
                s = ScreenRotationType.TYPE_LANDSCAPE
            }
            if (portrait.isChecked) {
                s = ScreenRotationType.TYPE_PORTRAIT
            }

            val rgm = dialogView.findViewById<RadioGroup>(R.id.rg_m)
            val rbA = rgm.findViewById<RadioButton>(R.id.rb_audio)
            val rbV = rgm.findViewById<RadioButton>(R.id.rb_video)
            val rbB = rgm.findViewById<RadioButton>(R.id.rb_audio_and_video)

            var sType = StreamType.TYPE_BOTH
            if (rbA.isChecked) {
                sType = StreamType.TYPE_AUDIO
            }
            if (rbV.isChecked) {
                sType = StreamType.TYPE_VIDEO
            }
            if (rbB.isChecked) {
                sType = StreamType.TYPE_BOTH
            }

            val etAutoRecycler = dialogView.findViewById<EditText>(R.id.et_auto_refcy_time)
            var autoRecyclerTime = "300"
            if (etAutoRecycler.text.isNotBlank()) {
                autoRecyclerTime = etAutoRecycler.text.toString().trim()
                if (autoRecyclerTime.toInt() <= 0 || autoRecyclerTime.toInt() > 7200) {
                    Toast.makeText(
                        this,
                        "自动无操作回收时间设置出错  范围: 1 ~ 7200",
                        Toast.LENGTH_SHORT
                    ).show()
                    return@setPositiveButton
                }
            }

            val userIdStr = userId.text.toString()
            val tokenStr = token.text.toString()
            val podIdStr = etPodId?.text.toString()
            val clientTypeStr = clientType.text.toString()

            if (TextUtils.isEmpty(userIdStr)) {
                Toast.makeText(this, "userId不能为空", Toast.LENGTH_SHORT).show()
                return@setPositiveButton
            }
            if (TextUtils.isEmpty(tokenStr)) {
                Toast.makeText(this, "token不能为空", Toast.LENGTH_SHORT).show()
                return@setPositiveButton
            }
            if (TextUtils.isEmpty(podIdStr)) {
                Toast.makeText(this, "podId不能为空", Toast.LENGTH_SHORT).show()
                return@setPositiveButton
            }
            if (TextUtils.isEmpty(clientTypeStr)) {
                Toast.makeText(this, "clientType不能为空", Toast.LENGTH_SHORT).show()
                return@setPositiveButton
            }

            listener.onPositiveClick(
                userIdStr,
                tokenStr,
                podIdStr,
                clientTypeStr,
                ve,
                s,
                sType,
                renderMode,
                rotation,
                autoRecyclerTime
            )
        }
        builder.setNegativeButton("取消") { dialog, which -> listener.onNegativeClick() }
        builder.show()
    }

    private fun saveImageToGallery(
        context: Context,
        bitmap: Bitmap,
        title: String,
        description: String
    ) {
        thread {
            val contentValues = ContentValues().apply {
                put(MediaStore.Images.Media.DISPLAY_NAME, title)
                put(MediaStore.Images.Media.DESCRIPTION, description)
                put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg")
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                    put(MediaStore.Images.Media.IS_PENDING, 1)
                }
            }
            val resolver = context.contentResolver
            val uri = resolver.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, contentValues)
            uri?.let { imageUri ->
                try {
                    val outputStream: OutputStream? = resolver.openOutputStream(imageUri)
                    outputStream?.use { bitmap.compress(Bitmap.CompressFormat.JPEG, 100, it) }
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                        contentValues.clear()
                        contentValues.put(MediaStore.Images.Media.IS_PENDING, 0)
                        resolver.update(imageUri, contentValues, null, null)
                    }
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            "已截图到相册",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                } catch (e: Exception) {
                    resolver.delete(imageUri, null, null)
                    e.printStackTrace()
                }
            }
        }
    }

    // 处理权限请求的回调
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String?>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        ArmCloudEngine.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    override fun onResume() {
        super.onResume()
        ArmCloudEngine.resume()
    }

    override fun onPause() {
        super.onPause()
        ArmCloudEngine.pause()
    }

    override fun onDestroy() {
        ArmCloudEngine.stop()
        super.onDestroy()
    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            val uri = data?.data
            val file = UriUtils.uri2File(uri)
            ArmCloudEngine.getUploadManager(application).uploadFile(
                uploadPodCode,
                uploadToken,
                file,
                uploadPath,
                object : IUploadFileCallBack {
                    override fun onUpdateStart() {
                        Log.d(TAG, "file upload  onUpdateStart ====> ${file.absolutePath}")
                    }

                    override fun onUpdateByteChange(totalByte: Long, updateByte: Long) {
                        //                    ArmLogUtils.d("file upload  onUpdateByteChange ====> ${file.absolutePath}")
                    }

                    override fun onUpdateProgressChange(progress: Int) {
                        Log.d(TAG, "file upload  onUpdateProgressChange ====> ${file.absolutePath} progress:$progress")
                    }

                    override fun onUpdateSuccess(mapData: Map<String, String>) {
                        Log.d(TAG, "file upload  onUpdateSuccess ====> ${file.absolutePath} fileId: ${mapData["fileUniqueId"]}")
                    }

                    override fun onUpdateFail(e: Throwable?) {
                        Log.d(TAG, "file upload  onUpdateFail ====> ${file.absolutePath} e: ${e?.message}")
                    }

                    override fun onUpdateEnd() {
                        Log.d(TAG,"file upload  onUpdateEnd ====> ${file.absolutePath}")
                    }

                    override fun onError(code: Int, msg: String) {
                        Log.d(TAG, "file upload  onError ====> ${file.absolutePath}  code: $code  msg: $msg")
                    }
                })
        }
    }
}